
def PRing(letter, n):
   names = [letter + str(k) for k in range(n)]
   return PolynomialRing(QQ, names)

def vars(letter, n):
   return PRing(letter, n).gens()

# test_elemz_odd

def mul_polynomial_by_binomial(acoefs, b):
   alen = len(acoefs)
   midcoefs = [acoefs[j] * b + acoefs[j - 1] for j in range(1, alen)]
   return [b * acoefs[0]] + midcoefs + [acoefs[alen - 1]]

def elem_pols(t):
   m = len(t)
   result = [1]
   for j in range(m):
      result = mul_polynomial_by_binomial(result, t[j])
   return list(reversed(result))

def test_elemz_odd_xpol(n, verbos):
   if verbos >= 2:
      print('test_elemz_odd_xpol, n = %d' % n)
   F = PRing('x', n)
   x = F.gens()
   y = list(x) + [1 / x[j] for j in range(n)] + [1]
   z = [x[j] + 1 / x[j] for j in range(n)]
   lhs = elem_pols(y)
   elem_pols_z = elem_pols(z) + [F.zero()] * (n + 1)
   rhs = [0] * (2 * n + 2)
   for m in range(2 * n + 2):
      kmin = max(0, 2 * m - 2 * n - 1)
      rhs[m] = sum(binomial(n - m + k, floor(k / 2)) * elem_pols_z[m - k] for k in range(kmin, m + 1))
   if verbos >= 3:
      print('lhs = ' + str(lhs))
      print('rhs = ' + str(rhs))
   return lhs == rhs

def big_test_elemz_odd_xpol(nmax, verbos):
   if verbos >= 1:
      print('big_test_elemz_odd_xpol, nmax = %d' % nmax)
   return all(test_elemz_odd_xpol(n, verbos) for n in range(1, nmax + 1))

# test_homz_odd

def prod_dif(x, j):
   return prod(x[j] - x[k] for k in range(j) + range(j + 1, len(x)))

def hom_via_geom_progr(x, m):
   n = len(x)
   return sum(x[j] ** (n + m - 1) / prod_dif(x, j) for j in range(n))

def homz(z, m):
   n = len(z)
   return sum(chebyshev_U(n + m - 1, z[j] / 2) / prod_dif(z, j) for j in range(n))

def coef_sum_chebU(m, k):
   jmax = floor((m - k) / 2)
   return sum(((- 1) ** j) * binomial(k + j, j) for j in range(jmax + 1))

def sum_chebU(m, t):
   return sum(coef_sum_chebU(m, k) * (t ** k) for k in range(m + 1))

def spz_odd_bialternant(z, la):
   F = z[0].parent()
   n = len(z)
   la_ext = la + [0] * (n - len(la))
   Mnumer = matrix(F, n, n)
   Mdenom = matrix(F, n, n)
   for j in range(n):
      for k in range(n):
         r = la_ext[j] + n - j - 1
         s = n - j - 1
         Mnumer[j, k] = sum_chebU(r, z[k])
         Mdenom[j, k] = sum_chebU(s, z[k])
   return det(Mnumer) / det(Mdenom)

def homz_odd(z, m):
   n = len(z)
   return sum(sum_chebU(n + m - 1, z[j]) / prod_dif(z, j) for j in range(n))

def homz_odd_as_sum_homz(z, m):
   return sum(homz(z, k) for k in range(m + 1))

def homz_odd_as_sum_hom(z, m):
   n = len(z)
   return sum(coef_sum_chebU(m + n - 1, k + n - 1) * hom_via_geom_progr(z, k) for k in range(m + 1))

def homz_odd_bialternant(z, m):
   la = [m]
   return spz_odd_bialternant(z, la)

def test_homz_odd_xpol(n, m, verbos):
   if verbos >= 2:
      print('test_homz_odd_xpol, n = %d, m = %d' % (n, m))
   F = PRing('x', n)
   x = F.gens()
   y = list(x) + [1 / x[j] for j in range(n)] + [1]
   z = [x[j] + 1 / x[j] for j in range(n)]
   res0 = hom_via_geom_progr(y, m)
   res1 = homz_odd(z, m)
   res2 = homz_odd_as_sum_homz(z, m)
   res3 = homz_odd_as_sum_hom(z, m)
   res4 = homz_odd_bialternant(z, m)
   if verbos >= 3:
      print('res0 = ' + str(res0))
      print('res1 = ' + str(res1))
      print('res2 = ' + str(res2))
      print('res3 = ' + str(res3))
      print('res4 = ' + str(res4))
   return (res0 == res1) and (res0 == res2) and (res0 == res3) and (res0 == res4)

def big_test_homz_odd_xpol(nmax, mmax, verbos):
   if verbos >= 1:
      print('big_test_homz_odd_xpol, nmax = %d' % nmax)
   tasks = [(n, m) for n in range(1, nmax + 1) for m in range(mmax + 1)]
   return all(test_homz_odd_xpol(n, m, verbos) for (n, m) in tasks)

def test_homz_odd_zpol(n, m, verbos):
   if verbos >= 2:
      print('test_homz_odd_zpol, n = %d, m = %d' % (n, m))
   F = PRing('z', n)
   z = F.gens()
   res0 = homz_odd(z, m)
   res1 = homz_odd_as_sum_homz(z, m)
   res2 = homz_odd_as_sum_hom(z, m)
   res3 = homz_odd_bialternant(z, m)
   if verbos >= 3:
      print('res0 = ' + str(res0))
      print('res1 = ' + str(res1))
      print('res2 = ' + str(res2))
      print('res3 = ' + str(res3))
   return (res0 == res1) and (res0 == res2) and (res0 == res3)

def big_test_homz_odd_zpol(nmax, mmax, verbos):
   if verbos >= 1:
      print('big_test_homz_odd_zpol, nmax = %d, mmax = %d' % (nmax, mmax))
   tasks = [(n, m) for n in range(1, nmax + 1) for m in range(mmax + 1)]
   return all(test_homz_odd_zpol(n, m, verbos) for (n, m) in tasks)

# test_powersumz_odd, m >= 1

def powersum(x, m):
   return sum(xelem ** m for xelem in x)

def mycoef1(m, k):
   s = (-1) ** k
   return s * m * binomial(m - k, k) / (m - k)

def powersumz_odd_via_sum_powersum(z, m):
   n = len(z)
   kmax = floor((m - 1) / 2)
   main_sum = sum(mycoef1(m, k) * powersum(z, m - 2 * k) for k in range(kmax + 1))
   last_summand = 2 * n * ((-1) ** (m / 2)) if mod(m, 2) == 0 else 0
   return 1 + main_sum + last_summand

def test_powersumz_odd_xpol(n, m, verbos):
   if verbos >= 2:
      print('test_powersumz_odd_xpol, n = %d, m = %d' % (n, m))
   F = PRing('x', n)
   x = F.gens()
   y = list(x) + [1 / x[j] for j in range(n)] + [F.one()]
   z = [x[j] + 1 / x[j] for j in range(n)]
   res0 = powersum(y, m)
   res1 = powersumz_odd_via_sum_powersum(z, m)
   if verbos >= 3:
      print('res0 = ' + str(res0))
      print('res1 = ' + str(res1))
   return res0 == res1

def big_test_powersumz_odd_xpol(nmax, mmax, verbos):
   if verbos >= 1:
      print('big_test_powersumz_odd_xpol, nmax = %d, mmax = %d' % (nmax, mmax))
   tasks = [(n, m) for n in range(1, nmax + 1) for m in range(1, mmax + 1)]
   return all(test_powersumz_odd_xpol(n, m, verbos) for (n, m) in tasks)

def all_small_tests_elemz_homz_powersumz_tests():
   verbos = 3
   res = [0] * 4
   res[0] = test_elemz_odd_xpol(1, verbos)
   res[1] = test_homz_odd_xpol(2, 3, verbos)
   res[2] = test_homz_odd_zpol(2, 3, verbos)
   res[3] = test_powersumz_odd_xpol(2, 3, verbos)
   total_result = all(res)
   print('total result = ' + str(total_result))

import time

def all_big_tests_elemz_homz_powersumz_odd_pol():
   verbos = 2
   res = [0] * 4
   t = time.time()
   res[0] = big_test_elemz_odd_xpol(12, verbos)
   res[1] = big_test_homz_odd_xpol(4, 8, verbos)
   res[2] = big_test_homz_odd_zpol(6, 10, verbos)
   res[3] = big_test_powersumz_odd_xpol(14, 20, verbos)
   total_result = all(res)
   t = time.time() - t
   print('time = %.2f seconds' % t)
   print('total result = ' + str(total_result))

# time < 200 sec
all_big_tests_elemz_homz_powersumz_odd_pol()

